import { describe, expect, it, jest, beforeEach, afterAll, afterEach } from '@jest/globals';
import request from "supertest";
import app from "../../app.js";
import salaRepository from '../../repository/salaRepository.js';
import salaServices from '../../services/salaServices.js';

jest.mock("../../repository/salaRepository.js", () => ({
    findAll: jest.fn(),
    create: jest.fn(),
    findById: jest.fn(),
    update: jest.fn(),
    delete: jest.fn(),
}));

let salaCriada = null

describe("Teste de integração de sala", () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    let token;
    console.log("Token:", token);

    it("Deve autenticar dape", async () => {
        const req = await request(app)
        .post('/authdape')
        .send({
            "n_siape_dape": "145732159",
            "senha_dape": "mara1234"
        })
        .set('Accept', 'aplication/json')
        .expect(200);
        token = req.body.data.token;
    });

    it("Deve cadastrar nova sala", async () => {
        const dados = await request(app)
        .post('/novasala')
        .send({
            nome_sala: "S2",
            bloco_sala: "A"
        })
        .set('Accept', 'application/json')
        .set("Authorization", `Bearer ${token}`)
        expect(dados.body.code).toBe(201);
        expect(dados.body.message).toBe("Requisição bem sucedida, recurso foi criado");
        expect(dados.body.error).toBe(false);
    });

    it("Deve listar sala", async () => {

        const mockSalas = [
            { id_sala: 1, nome_sala: "S1", bloco_sala: "A" },
            { id_sala: 2, nome_sala: "S2", bloco_sala: "B" }
        ];

        jest.spyOn(salaRepository, 'findAll').mockResolvedValue(mockSalas);

        const req = await request(app)
        .get('/sala')
        .set('Accept', 'application/json')
        .set("Authorization", `Bearer ${token}`)
        expect(req.body.error).toEqual(false)
        expect(req.status).toBe(200)
        expect(req.body.message).toEqual("Salas encontradas com sucesso!")
        expect(req.body.data).toBeInstanceOf(Array)
        expect(req.body.data.length).toBeGreaterThan(0)
        expect(req.body.data[0].id_sala).toBeDefined()
        expect(req.body.data[0].nome_sala).toBeDefined()
        expect(req.body.data[0].bloco_sala).toBeDefined()
        // idSala=dados.body[0].id_sala;
    });

    // it.skip("Deve listar sala por id", async () => {
    //     const req = await request(app)
    //     .get(`/sala/${idSala}`)
    //     .expect(200);
    // });

    it("Deve atualizar sala por id", async () => {
        const id = 80;
        const updateData = {
            nome_sala: "S2",
            bloco_sala: "B"
        }
        const req = await request(app)
        .patch(`/sala/${id}`)
        .set('Accept', 'application/json')
        .set("Authorization", `Bearer ${token}`)
        .send(updateData);
        console.log(req.body);
        expect(req.body.error).toEqual(false)
        expect(req.status).toBe(200);
        expect(req.body.message).toMatch("Requisição bem sucedida.");
        expect(req.body.data).toHaveProperty('nome_sala', updateData.nome_sala);
        expect(req.body.data).toHaveProperty('bloco_sala', updateData.bloco_sala);
    });

    it("Deve lançar erro se a sala não existir", async () => {
        const id = { id_sala: '999' };
        const data = { nome_sala: 'S1', bloco_sala: "B" };

        salaRepository.findById.mockResolvedValue(null);

        await expect(salaServices.atualizar(id, data)).rejects.toEqual({
            message: `Sala com id 999 não existe!`,
            code: 404
        });
    });

    it("Deve excluir sala por id", async () => {
        console.log(salaCriada)
        const id = salaCriada.id_sala;

        const response = await request(app)
        .delete(`/sala/${id}`)
        .set("Authorization", `Bearer ${token}`)
        .set("Content-Type", "application/json")
        expect(response.status).toBe(200);
        expect({ error: false }).toHaveProperty('error', false);
        
    });

    it("Deve lançar erro se a sala não existir", async () => {
        const id = { id_sala: '999' };

        salaRepository.findById.mockResolvedValue(null);

        await expect(salaServices.excluir(id)).rejects.toEqual({
            message: `Sala com id 999 não existe!`,
            code: 404
        });
    });
})