import { describe, expect, it, jest, beforeEach, test } from '@jest/globals';
import dapeServices from "../../services/dapeServices.js";
import dapeRepository from "../../repository/dapeRepository.js"

jest.mock("../../repository/dapeRepository.js", () => ({
    findAll: jest.fn(),
    create: jest.fn(),
    findById: jest.fn(),
    update: jest.fn(),
    delete: jest.fn(),
    dapeExist: jest.fn(),
}));

describe("Teste unitário de dape", () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('Deve retornar todos os usuários dape', async () => {
        const mockDape =
            {
                n_siape_dape: "114787412032", 
                nome_dape: "Maria Souza", 
                email_dape: "mariasouza@gmail.com", 
                senha_dape: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
                tipo_usuario: "dape"
            }

        dapeRepository.findAll.mockResolvedValue(mockDape);

        const dape = await dapeServices.listar({})

        expect(dape).toEqual(mockDape);
    });

    it("Deve retornar usuário dape por id", async () => {
        const mockDape = 
        {
            n_siape_dape: "11077412032", 
            nome_dape: "Mara Silva", 
            email_dape: "marasilva458@gmail.com",
            senha_dape: "$2a$10$A8EzRdTmos8ukokxwD3L1.Z4o5T7YAJzWedRRi1U4xQ/b4E1uAli.",
            tipo_usuario: "dape", 
            fotoPerfil: ""
        };
        dapeRepository.findById.mockResolvedValue(mockDape);

        const dape = await dapeServices.listarPorId({n_siape_dape: "11077412032"});

        expect(dape).toEqual(mockDape);
        expect(dapeRepository.findById).toHaveBeenCalledWith({where: {n_siape_dape: "11077412032"}});
    })

    it("Deve cadastrar novo usuário dape", async () => {
        const mockDape = 
        {
            n_siape_dape: "5646548944", 
            nome_dape: "José Silva", 
            email_dape: "josesilva@gmail.com", 
            senha_dape: "jose1245678", 
            tipo_usuario: "dape"
        };

        dapeRepository.findById.mockResolvedValue(null);
        dapeRepository.dapeExist.mockResolvedValue(null);

        dapeRepository.create.mockResolvedValue(mockDape);

        const dape = await dapeServices.inserir(mockDape);

        expect(dape).toEqual(mockDape);
        expect(dapeRepository.create).toHaveBeenCalled();

    });

    it("Deve atualizar usuário dape por id", async () => {
        const mockExistingDape = 
        {
            n_siape_dape: "11077412032", 
            nome_dape: "Mara Silva", 
            email_dape: "marasilva458@gmail.com", 
            senha_dape: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
            tipo_usuario: "dape"
        };

        dapeRepository.findById.mockResolvedValue(mockExistingDape);

        const mockDape = 
        {
            nome_dape: "José Silva", 
        };

        dapeRepository.update.mockResolvedValue(mockDape);

        const updateDape = await dapeServices.atualizar({n_siape_dape: "11077412032"}, mockDape);

        expect(updateDape).toEqual(mockDape);
        expect(dapeRepository.findById).toHaveBeenCalledWith({where: {n_siape_dape: "11077412032"}});
        expect(dapeRepository.update).toHaveBeenCalledWith("11077412032", mockDape);


    });

    it("Deve validar se usuário dape existe quando atualizar", async () => {
        const mockDape = {
            n_siape_dape: "11077412032",
            nome_dape: "José Silva",
            email_dape: "josesilva@gmail.com",
            senha_dape: "jose1245678",
            tipo_usuario: "dape"
        };

        dapeRepository.findById.mockResolvedValue(false);

        await expect(dapeServices.atualizar({n_siape_dape: "11077412032"}, mockDape))
            .rejects
            .toEqual({
                message: "Dape com Número do siape 11077412032 não existe!",
                code: 404
            });
    
        expect(dapeRepository.findById).toHaveBeenCalledWith({where: {n_siape_dape: "11077412032"}});
        expect(dapeRepository.update).not.toHaveBeenCalled();
        
    });

    it("Deve excluir usuário dape por id", async () => {
        const mockDape = 
        {
            n_siape_dape: "11077412032", 
            nome_dape: "José Silva", 
            email_dape: "josesilva@gmail.com", 
            senha_dape: "jose1245678", 
            tipo_usuario: "dape"
        };

        dapeRepository.findById.mockResolvedValue(mockDape);
        dapeRepository.delete.mockResolvedValue(mockDape);

        const deleteDape = await dapeServices.excluir({n_siape_dape: "11077412032"});

        expect(deleteDape).toEqual(mockDape);
        expect(dapeRepository.findById).toHaveBeenCalledWith({where: {n_siape_dape: "11077412032"}});
        expect(dapeRepository.delete).toHaveBeenCalledWith("11077412032");
    });
    
    it("Deve validar se usuário dape existe quando excluir", async () => {
        const mockDape = {
            n_siape_dape: "11077412032",
            nome_dape: "José Silva",
            email_dape: "josesilva@gmail.com",
            senha_dape: "jose1245678",
            tipo_usuario: "dape"
        };

        dapeRepository.findById.mockResolvedValue(false);

        await expect(dapeServices.excluir({n_siape_dape: "11077412032"}, mockDape))
            .rejects
            .toEqual({
                message: "Dape com Número do siape 11077412032 não existe!",
                code: 404
            });
    
        expect(dapeRepository.findById).toHaveBeenCalledWith({where: {n_siape_dape: "11077412032"}});
        expect(dapeRepository.delete).not.toHaveBeenCalled();
    })
});