import { describe, expect, it, jest, beforeEach, test } from '@jest/globals';
import docenteServices from "../../services/docenteServices.js";
import docenteRepository from "../../repository/docenteRepository.js"

jest.mock("../../repository/docenteRepository.js", () => ({
    findAll: jest.fn(),
    create: jest.fn(),
    findById: jest.fn(),
    update: jest.fn(),
    delete: jest.fn(),
    docenteExist: jest.fn(),
}));

describe("Teste unitário de docente", () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('Deve retornar todos os usuários docente', async () => {
        const mockDocente =
            {   n_siape: "123456789", 
                nome_docente: "Antony Silva", 
                email_docente: "antonysilva@gmail.com", 
                senha_docente: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
                tipo_usuario: "docente"

            };

        docenteRepository.findAll.mockResolvedValue(mockDocente);

        const docente = await docenteServices.listar({})

        expect(docente).toEqual(mockDocente);
        
    });

    it("Deve retornar usuário docente por id", async () => {
        const mockDocente =
            {
                n_siape: "123456789", 
                nome_dape: "Antony Silva", 
                email_dape: "antonysilva@gmail.com",
                senha_docente: "$2a$10$A8EzRdTmos8ukokxwD3L1.Z4o5T7YAJzWedRRi1U4xQ/b4E1uAli.",
                turma_docente: "1B e 2A",
                curso_docente: "Informática",
                tipo_usuario: "docente", 
                fotoPerfil: ""
            };

        docenteRepository.findById.mockResolvedValue(mockDocente);

        const docente = await docenteServices.listarPorId({n_siape: "123456789"});

        expect(docente).toEqual(mockDocente);
        expect(docenteRepository.findById).toHaveBeenCalledWith({ where: { n_siape: "123456789" } });

    });

    it("Deve cadastrar novo usuário docente", async () => {
        const mockDocente =
            {
                n_siape: "1487654301", 
                nome_docente: "Antonio Souza", 
                email_docente: "antoniosouza@gmail.com", 
                senha_docente: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum",
                turma_docente: "2 B",
                curso_docente: "Eletromecânica", 
                tipo_usuario: "docente"
            };

        docenteRepository.findById.mockResolvedValue(null);
        docenteRepository.docenteExist.mockResolvedValue(null);

        docenteRepository.create.mockResolvedValue(mockDocente);
        const docente = await docenteServices.inserir(mockDocente);

        expect(docente).toEqual(mockDocente);
        expect(docenteRepository.create).toHaveBeenCalled();

    });

    it("Deve atualizar usuário docente por id", async () => {
        const mockDocenteExiste =
            {
                n_siape: "123456789", 
                nome_docente: "Antony Silva", 
                email_docente: "antonysilva@gmail.com", 
                senha_docente: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
                tipo_usuario: "dape"
            };

        docenteRepository.findById.mockResolvedValue(mockDocenteExiste);

        const mockDocente =
            {
                nome_docente: "Antonio Souza", 
                email_docente: "antoniosouza@gmail.com",
            };

        docenteRepository.update.mockResolvedValue(mockDocente);

        const updateDocente = await docenteServices.atualizar({n_siape: "123456789"}, mockDocente);

        expect(updateDocente).toEqual(mockDocente);
        expect(docenteRepository.findById).toHaveBeenCalledWith({where: {n_siape: "123456789"}});
        expect(docenteRepository.update).toHaveBeenCalledWith("123456789", mockDocente);
    
    });

    it("Deve validar se usuário docente existe quando atualizar", async () => {
        const mockDocente =
            {
                n_siape: "123456789", 
                nome_docente: "Antony Silva", 
                email_docente: "antonysilva@gmail.com", 
                senha_docente: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
                tipo_usuario: "docente"
            };

        docenteRepository.findById.mockResolvedValue(false);

        await expect(docenteServices.atualizar({ n_siape: "123456789" }, mockDocente)).rejects.toEqual(
            {
                message: "Docente com Número do siape 123456789 não existe!",
                code: 404
            });
    
        expect(docenteRepository.findById).toHaveBeenCalledWith({where: {n_siape: "123456789"}});
        expect(docenteRepository.update).not.toHaveBeenCalled();
        
    });

    it("Deve excluir usuário docente por id", async () => {
        const mockDocente =
            {
                n_siape: "123456789", 
                nome_docente: "Antony Silva", 
                email_docente: "antonysilva@gmail.com", 
                senha_docente: "$2a$10$.yY5zliayVHk1BLN294yvO/Mi60pkUqa7d7tfKkwAYiS0wG71nKum", 
                tipo_usuario: "docente"
            };

        docenteRepository.findById.mockResolvedValue(mockDocente);
        docenteRepository.delete.mockResolvedValue(mockDocente);

        const deleteDocente = await docenteServices.excluir({n_siape: "123456789"});

        expect(deleteDocente).toEqual(mockDocente);
        expect(docenteRepository.findById).toHaveBeenCalledWith({where: {n_siape: "123456789"}});
        expect(docenteRepository.delete).toHaveBeenCalledWith("123456789");
    });

    it("Deve validar se usuário docente existe quando excluir", async () => {
        const mockDocente =
            {
                n_siape: "123456789", 
                nome_docente: "Antony Silva", 
                email_docente: "antonysilva@gmail.com", 
                senha_docente: "123456789", 
                tipo_usuario: "docente"
            } 
        docenteRepository.findById.mockResolvedValue(false);
    
        await expect(docenteServices.excluir({n_siape: "123456789"}, mockDocente)).rejects.toEqual(
            {
                message: "Docente com Número do siape 123456789 não existe!", 
                code: 404
            });
    
        expect(docenteRepository.findById).toHaveBeenCalledWith({where: {n_siape: "123456789"}});
        expect(docenteRepository.delete).not.toHaveBeenCalled();
        
    })
});