import { describe, expect, it, jest, beforeEach } from '@jest/globals';
import salaServices from '../../services/salaServices';
import salaRepository from '../../repository/salaRepository';

jest.mock('../../repository/salaRepository', () => ({
    findAll: jest.fn(),
    findById: jest.fn(),
    create: jest.fn(),
    update: jest.fn(),
    delete: jest.fn(),
}));

describe('Teste unitário de sala', () => {
    beforeEach(() => {
        jest.clearAllMocks();
    });

    it('Deve retornar todas as salas', async () => {
        const mockSalas = [
            {id_sala: 7, bloco_sala: 'B', nome_sala: 'T1'},
            {id_sala: 9, bloco_sala: 'B', nome_sala: 'T2'},
        ];

        salaRepository.findAll.mockResolvedValue(mockSalas);
        const sala = await salaServices.listar({})

        expect(sala).toEqual(mockSalas);
        expect(salaRepository.findAll).toHaveBeenCalled();
    });

    it('Deve criar uma nova sala', async () => {
        const mockSalasCreate = 
        {
            id_sala: 1, 
            bloco_sala: 'B', 
            nome_sala: 'T1'
        }


        salaRepository.create.mockResolvedValue(mockSalasCreate);
        const salaCreate = await salaServices.inserir(mockSalasCreate)

        expect(salaCreate).toEqual(mockSalasCreate);
        expect(salaRepository.create).toHaveBeenCalled();
    });

    it('Deve atualizar as salas', async () => {
        const mockSalasUpdate = 
            {
                id_sala: 2, 
                bloco_sala: 'C', 
                nome_sala: 'T4'
            }
        const mockSalasExiste = 
        {
            id_sala: 2, 
            bloco_sala: 'C', 
            nome_sala: 'T2'
        }

        salaRepository.findById.mockResolvedValue(mockSalasExiste);
        salaRepository.update.mockResolvedValue(mockSalasUpdate);

        const salaUpdate = await salaServices.atualizar({id_sala: 2}, mockSalasUpdate);

        expect(salaUpdate).toEqual(mockSalasUpdate);
        expect(salaRepository.findById).toHaveBeenCalledWith(2);
        expect(salaRepository.update).toHaveBeenCalled();
    });

    it('Deve validar se sala existe quando atualizar', async () => {
    
    const mockSalaUpdate =
        {
            id_sala: 3, 
            bloco_sala: 'F', 
            nome_sala: 'T4'
        }

    salaRepository.findById.mockResolvedValue(false);

    await expect(salaServices.atualizar({id_sala: 3}, mockSalaUpdate)).rejects.toEqual(
        { 
            message: `Sala com id 3 não existe!`, 
            code: 404
        });
    
        expect(salaRepository.findById).toHaveBeenCalledWith(3);
        expect(salaRepository.update).not.toHaveBeenCalled();
    });

    it("Deve excluir uma sala por ID", async () => {
        const mockSalaDelete =
            {
                id_sala: 1, 
                bloco_sala: 'B', 
                nome_sala: 'T2'
            };

        salaRepository.findById.mockResolvedValue(mockSalaDelete);
        salaRepository.delete.mockResolvedValue(mockSalaDelete);

        const salaDelete = await salaServices.excluir({id_sala: 1});

        expect(salaDelete).toEqual(mockSalaDelete);
        expect(salaRepository.findById).toHaveBeenCalledWith(1);
        expect(salaRepository.delete).toHaveBeenCalledWith(1);
    });

    it("Deve validar se sala existe quando excluir", async () => {
        const mockSalaDelete =
            {
                id_sala: 2, 
                bloco_sala: 'B', 
                nome_sala: 'T2'
            }

        salaRepository.findById.mockResolvedValue(false);

        await expect(salaServices.excluir({id_sala: 2}, mockSalaDelete)).rejects.toEqual(
            {
                message: "Sala com id 2 não existe!",
                code: 404
            });

        expect(salaRepository.findById).toHaveBeenCalledWith(2);
        expect(salaRepository.delete).not.toHaveBeenCalled();
    })
});